unit fEditReg;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, ORSystem,
  StdCtrls, FileCtrl, ExtCtrls, ORFn;

type
  TfrmEditReg = class(TForm)
    Label1: TLabel;
    txtGoldCopy: TEdit;
    chkAutoUpdate: TCheckBox;
    dlgOpen: TOpenDialog;
    cmdBrowse: TButton;
    cmdOK: TButton;
    cmdCancel: TButton;
    txtLimitPath: TEdit;
    Label2: TLabel;
    Button1: TButton;
    pnlCopy: TPanel;
    chkAskFirst: TCheckBox;
    timCopy: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure cmdBrowseClick(Sender: TObject);
    procedure cmdCancelClick(Sender: TObject);
    procedure cmdOKClick(Sender: TObject);
    procedure timCopyTimer(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmEditReg: TfrmEditReg;

implementation

{$R *.DFM}

uses ShellAPI;

const
  DELAY_SCREEN = 2000;

procedure BuildCommandLine(var x: string);
var
  i: Integer;
begin
  for i := 1 to ParamCount do
  begin
    if Copy(UpperCase(ParamStr(i)), 1, Pos('=', ParamStr(i)) - 1) = 'GOLD' then Continue;
    if Copy(UpperCase(ParamStr(i)), 1, Pos('=', ParamStr(i)) - 1) = 'AUTO' then Continue;
    if Copy(UpperCase(ParamStr(i)), 1, Pos('=', ParamStr(i)) - 1) = 'COPY' then Continue;
    if Copy(UpperCase(ParamStr(i)), 1, Pos('=', ParamStr(i)) - 1) = 'XFER' then Continue;
    if Copy(UpperCase(ParamStr(i)), 1, Pos('=', ParamStr(i)) - 1) = 'ONLY' then Continue;
    if Copy(UpperCase(ParamStr(i)), 1, Pos('=', ParamStr(i)) - 1) = 'SHOW' then Continue;
    if Copy(UpperCase(ParamStr(i)), 1, Pos('=', ParamStr(i)) - 1) = 'ASK'  then Continue;
    if Copy(UpperCase(ParamStr(i)), 1, Pos('=', ParamStr(i)) - 1) = 'RUN'  then Continue;
    x := x + ' ' + ParamStr(i);
  end;
end;

function EnsureBackSlash(const x: string): string;
begin
  Result := x;
  if (Length(x) > 0) and (x[Length(x)] <> '\') then Result := x + '\';
end;

procedure UpdateAppFromGold(AppName: string);
var
  CopyOK: Boolean;
  GoldName, BackName, ATitle, SHGold, SHApp: string;
  FileOp: TSHFileOpStruct;
begin
  GoldName := RegReadStr(CPRS_REG_GOLD) + FullToFilePart(AppName);
  if (Length(GoldName) > 0) and FileExists(GoldName) then
  begin
    // make backup copy of app
    if Pos('.exe', LowerCase(AppName)) > 0
      then BackName := Copy(AppName, 1, Pos('.exe', LowerCase(AppName)) - 1) + '.bak'
      else BackName := AppName + '.bak';
    if FileExists(BackName) then SysUtils.DeleteFile(BackName);
    SysUtils.RenameFile(AppName, BackName);
    // copy file
    ATitle := 'Updating CPRS: ' + AppName;
    if IsWin95Style then
    begin
      SHGold := GoldName + #0 + #0;
      SHApp  := AppName  + #0 + #0;
      FillChar(FileOp, SizeOf(FileOp), #0);
      with FileOp do
      begin
        Wnd   := Application.MainForm.Handle;
        wFunc := FO_COPY;
        pFrom := PChar(SHGold);
        pTo   := PChar(SHApp);
        fFlags := FOF_NOCONFIRMATION or FOF_SIMPLEPROGRESS;
        lpszProgressTitle := PChar(ATitle);
      end;
      CopyOK := (SHFileOperation(FileOp) = 0) and (FileOp.fAnyOperationsAborted = False);
    end
    else CopyOK := CopyFile(PChar(GoldName), PChar(AppName), False);   // for NT 3.5
    Delay(500);  // delay gives windows a chance to finish closing files
    // compare version numbers
    if CopyOK and (ClientVersion(AppName) <> ClientVersion(GoldName)) then CopyOK := False;
    if CopyOK then
    begin
      // delete the backup and make sure file dates match if copy succeeded
      Delay(500);
      SysUtils.DeleteFile(BackName);
      CopyFileDate(GoldName, AppName);
    end else
    begin
      // restore original file because the copy failed
      Delay(500);
      SysUtils.DeleteFile(AppName);
      SysUtils.RenameFile(BackName, AppName);
    end;
  end;
  // moved the recording of update time to timTimer so set even when not updating
end;

procedure TfrmEditReg.FormCreate(Sender: TObject);
var
  x: string;
  EndApp: Boolean;
begin
  EndApp := True;
  // set registry for location of gold copy
  if ParamIndex('GOLD') > 0
    then RegWriteStr(CPRS_REG_GOLD, EnsureBackSlash(ParamSearch('GOLD')));
  // set registry for limitupdate directory
  if ParamIndex('ONLY') > 0
    then RegWriteStr(CPRS_REG_ONLY, EnsureBackSlash(ParamSearch('ONLY')));
  // set registry for auto-update flag
  x := ParamSearch('AUTO');
  if (x = 'ON')  or (x = 'TRUE')  or (x = 'YES') then RegWriteBool(CPRS_REG_AUTO, True);
  if (x = 'OFF') or (x = 'FALSE') or (x = 'NO')  then RegWriteBool(CPRS_REG_AUTO, False);
  // set registry for ask user first flag
  x := ParamSearch('ASK');
  if (x = 'ON')  or (x = 'TRUE')  or (x = 'YES') then RegWriteBool(CPRS_REG_ASK, True);
  if (x = 'OFF') or (x = 'FALSE') or (x = 'NO')  then RegWriteBool(CPRS_REG_ASK, False);

  (*
  // check to see if need to run application
  AppName := ParamSearch('RUN');
  if (Length(AppName) > 0) and AppOutOfDate(AppName) then UpdateAppFromGold(AppName);
  *)

  // check for parameter that allows interactive setting of the registry settings
  if ParamIndex('SHOW') > 0 then
  begin
    EndApp := False;
    pnlCopy.Visible := False;
    txtGoldCopy.Text      := RegReadStr(CPRS_REG_GOLD);
    txtLimitPath.Text     := RegReadStr(CPRS_REG_ONLY);
    chkAutoUpdate.Checked := RegReadBool(CPRS_REG_AUTO);
    chkAskFirst.Checked   := RegReadBool(CPRS_REG_ASK);
    WindowState := wsNormal;
  end;
  // set up for copying application, if COPY param present
  if Length(ParamSearch('COPY')) > 0 then
  begin
    EndApp := False;
    pnlCopy.Visible := True;
    pnlCopy.BringToFront;
    WindowState := wsNormal;
    timCopy.Tag := 1;
    timCopy.Enabled := True;
  end;
  // set up to copy files, if XFER param present
  x := ParamSearch('XFER');
  if Length(x) > 0 then
  begin
    EndApp := False;
    pnlCopy.Visible := True;
    pnlCopy.BringToFront;
    timCopy.Tag := 2;
    timCopy.Enabled := True;
  end;
  if EndApp then Application.Terminate;
end;

procedure TfrmEditReg.timCopyTimer(Sender: TObject);
const
  TC_NEW_APP = 'Update Application';
  TX_NEW_APP = CRLF + CRLF + 'A new version of this application is available.' + CRLF +
               'Should this update be performed now?';
var
  CopyName, x: string;
  MakeCopy: Boolean;
  Millisecs: Integer;
begin
  timCopy.Enabled := False;
  Millisecs := GetTickCount;
  MakeCopy := True;
  // Tag=1 means copy, copy = copy & run, xfer = just copy (as in help files)
  if timCopy.Tag = 1 then CopyName := ParamSearch('COPY') else CopyName := ParamSearch('XFER');
  // check to see if in directory that should be updated
  x := RegReadStr(CPRS_REG_ONLY);
  if (Length(x) > 0) and (CompareText(x, FullToPathPart(CopyName)) <> 0) then MakeCopy := False;
  // check to see if should ask before updating
  x := CopyName + TX_NEW_APP;
  if MakeCopy and (timCopy.Tag = 1) and RegReadBool(CPRS_REG_ASK) and
    (Application.MessageBox(PChar(x), TC_NEW_APP, MB_YESNO) <> IDYES)    then MakeCopy := False;
  // record time the application attempted update (to prevent loop trying to update)
  UserRegWriteDateTime(CPRS_LAST_DATE, FullToFilePart(CopyName), Now);
  UserRegWriteInt(CPRS_USER_LAST, FullToFilePart(CopyName), GetTickCount);
  // do the actual copying
  if MakeCopy and (Length(CopyName) > 0) then UpdateAppFromGold(CopyName);
  // now run the copied program
  if timCopy.Tag = 1 then
  begin
    BuildCommandLine(CopyName);
    if Length(CopyName) > 0 then RunProgram(CopyName);
    Millisecs := Integer(GetTickCount) - Millisecs;
    if Millisecs < DELAY_SCREEN then Delay(DELAY_SCREEN - Millisecs);
  end;
  Close;
end;

procedure TfrmEditReg.cmdBrowseClick(Sender: TObject);
var
  ADir: string;
begin
  ADir := '';
  if SelectDirectory(ADir, [], 0) then with Sender as TWinControl do
  begin
    if Tag = 1 then txtGoldCopy.Text  := ADir;
    if Tag = 2 then txtLimitPath.Text := ADir;
  end;
end;

procedure TfrmEditReg.cmdOKClick(Sender: TObject);
var
  x: string;
begin
  // update registry
  x := EnsureBackSlash(txtGoldCopy.Text);
  RegWriteStr(CPRS_REG_GOLD, x);
  x := EnsureBackSlash(txtLimitPath.Text);
  RegWriteStr(CPRS_REG_ONLY, x);
  RegWriteBool(CPRS_REG_AUTO, chkAutoUpdate.Checked);
  RegWriteBool(CPRS_REG_ASK,  chkAskFirst.Checked);
  Close;
end;

procedure TfrmEditReg.cmdCancelClick(Sender: TObject);
begin
  Close;
end;

end.

